/*EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE

            Program to estimate a FSMS-RSDC model as in 
E.OTRANTO (2015): "Adding Flexibility to Markov Switching Models" CRENoS WP 2015/09

The model requires a preliminary estimation of nv univariate GARCH(1,1) models.
The input files of this program are:
res_dj.out: contains the standardized residuals of the Asymmetric GARCH(1,1) models
corr_dj.out: contains the the correlation matrix of the standardized residuals



                          Author: EDOARDO OTRANTO
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE*/

new;
/******************
LOAD THE SERIES
******************/
t=3541;@length of the data set@
nv=28;
l=1;
load SC[nv,nv]=c:\users\edoardo\dropbox\mio\fms\codes\corr_dj.out;
load U[t,nv]=c:\users\edoardo\dropbox\mio\fms\codes\res_dj.out;
t=3272;
U=U[1:t,.]; @length of the series analyzed@

pmax=9;
vsc=vech(sc);
vsc=delif(vsc,vsc.==1);
mvs=meanc(vsc);
vsc=maxc(vsc);
RH=ones(nv,nv)*mvs;
RH=diagrv(RH,ones(nv,1));
/*******************************************************************************************************************************************
NOTE: IF YOU WANT TO CONSTRAINT SOME COEFFICIENT TO BE 0, DO NOT CONSIDER IT IN THE FOLLOWING VECTOR, 
AND FIX ITS POSITION IN THE VECTOR HOLES. PUT HOLES=0 IF NO CONSTRAINTS ARE IMPOSED 
*******************************************************************************************************************************************/
@starting values@
x=-0.73253277|
2.0787347|
-4.5691775|
-0.9520274|
7.042394|
3.9061819|
5.4017054|
0.33262378|
0.11251908
; 
     

holes=0;
rh=rows(holes);


/*=======================================================================*/
 @                  create the sequence of states                             @
/*=======================================================================*/
state=zeros(2^l,1);
if l>0;
state=zeros(2^l,l);
endif;
j=1;
   do while j<=l;
   i=1;
      do while i<=(2^j);
      state[(i-1)*2^(l-j)+1:i*2^(l-j),j]=(i-int(i/2)*2)*ones(2^(l-j),1);
      i=i+1;
      endo;
   j=j+1;
   endo;
i=1;
   do while i<=(2^l);
   j=1;
       do while j<=l;
          if state[i,j]==0;
          state[i,j]=2;
          endif;
       j=j+1;
       endo;
   i=i+1;
   endo;


/**************************************************************************
                        estimation
**************************************************************************/
last=rows(x);
n=2;
library optmum;
{x,likv,gr,err}=optmum(&ll,x);
{likv,sp,fp,fp1}=ls(x);
xr=fp1[1,.]';
hout0=(hessp(&ll,x)); @calculate standard errors@
x0=trans(x);
x0=constr(x0);
p00=x0[1];
p01=1-p00;
p11=x0[2];
p10=1-p11;
PS=p00~p01|p10~p11;
lambda=x0[3:5];
gams=x0[6:7];
cp=x0[8:9];


I_op=zeros(rows(x),rows(x));
tim=2;
   do while tim<=t;
   gr_sc=gradp(&sc_ll,x);
   I_op=I_op+gr_sc'gr_sc;
   tim=tim+1;
   endo;
V_qm=inv(hout0*inv(I_op)*hout0);
grdn=gradfd(&TRANS,x);
cov=grdn*V_qm*grdn';
se =sqrt(diag(cov)); @Standard errors of the estimated coefficients@


OUTPUT FILE=EST.OUT RESET;
"estimates of parameters and corresponding standard errors:" 
x0;se;
OUTPUT OFF;
gs=zeros(t,2);
tim=1;
   do while tim<=t;
   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   gs[tim,.]=gs1~gs2;
   tim=tim+1;
   endo;

/***************************************************************************
              PROCEDURE TO Evaluate THE LOG-LIKELIHOOD and the smoothing probabilities
****************************************************************************/
proc(4)=ls(x);
local x0,p00,p01,
p11,p10,ps,D,in,eta,tim,r,snum,fy,ll,s1,s2,s3,s3s,lam,lambda,gs1,gs2,gams,cp,xr;

ll=0;
x0=trans(x); 
x0=constr(x0);


p00=x0[1];
p01=1-p00;
p11=x0[2];
p10=1-p11;
PS=p00~p01|p10~p11;
lambda=x0[3:5];
gams=x0[6:7];
cp=x0[8:9];



s1=zeros(n,t);   
s2=zeros(n,t);   
xr=zeros(t,1);



PS=p00~p01|
p10~p11;




D=(eye(n)-ps')|ones(1,n);
in=zeros(n,1)|1;
s2[.,2]=inv(D'D)*D'in;
eta=zeros(n,1);


tim=2;
   do while tim<=t-1;
   xr[tim]=s2[1,tim];
   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   lam=lambda[1]*gs1+lambda[2]*(1-gs1);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[1]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));
   lam=gs2+lambda[3]*(1-gs2);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[2]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));




   snum=s2[.,tim].*eta;
   fy=ones(1,n)*snum;
   ll=ll+ln(fy);
   s1[.,tim]=snum/fy;
  s2[.,tim+1]=ps'*s1[.,tim];
   tim=tim+1;
   endo;
   xr[tim]=s2[1,tim];
   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   lam=lambda[1]*gs1+lambda[2]*(1-gs1);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[1]=exp(-.5*(ln(det(R))+U[t,.]*inv(R)*U[t,.]'@-U[tim,.]*U[tim,.]'@));
   lam=gs2+lambda[3]*(1-gs2);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[2]=exp(-.5*(ln(det(R))+U[t,.]*inv(R)*U[t,.]'@-U[tim,.]*U[tim,.]'@));
snum=s2[.,t].*eta;
fy=ones(1,n)*snum;
s1[.,t]=snum/fy;

s3=zeros(n,t);   
s3[.,t]=s1[.,t];
tim=t-1;
   do while tim>=2;
  
   s3[.,tim]=s1[.,tim].*(PS*(s3[.,tim+1]./s2[.,tim+1]));   
   tim=tim-1;
   endo;
s3s=s3';

retp(-ll,s3s,s1,s2);
endp; 

/***************************************************************************
              PROCEDURE TO EXPLICIT THE LOG-LIKELIHOOD 
****************************************************************************/
proc(1)=ll(x);
local x0,a1,b1,a2,b2,p00,p01,p11,
p10,ps,D,in,s,eta,tim,r,snum,fy,ll,lambda,gs1,gs2,gams,cp,lam,xr;

ll=0;
x0=trans(x); 
x0=constr(x0);
p00=x0[1];
p01=1-p00;
p11=x0[2];
p10=1-p11;
PS=p00~p01|p10~p11;
lambda=x0[3:5];
gams=x0[6:7];
cp=x0[8:9];




D=(eye(n)-ps')|ones(1,n);
in=zeros(n,1)|1;
s=inv(D'D)*D'in;
eta=zeros(n,1);
xr=zeros(t,1);
tim=2;
   do while tim<=t-1;
   xr[tim]=s[1];
   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   lam=lambda[1]*gs1+lambda[2]*(1-gs1);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[1]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));
   lam=gs2+lambda[3]*(1-gs2);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[2]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));




   snum=s.*eta;
   fy=ones(1,n)*snum;
   ll=ll+ln(fy);
   s=snum/fy;
  s=ps'*s;
   tim=tim+1;
   endo;
   xr[tim]=s[1];
   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   lam=lambda[1]*gs1+lambda[2]*(1-gs1);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[1]=exp(-.5*(ln(det(R))+U[t,.]*inv(R)*U[t,.]'@-U[tim,.]*U[tim,.]'@));
   lam=gs2+lambda[3]*(1-gs2);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[2]=exp(-.5*(ln(det(R))+U[t,.]*inv(R)*U[t,.]'@-U[tim,.]*U[tim,.]'@));




   snum=s.*eta;
   fy=ones(1,n)*snum; 
   ll=ll+ln(fy);
retp(-ll);
endp; 


/***************************************************************************
              PROCEDURE TO EXPLICIT THE SCORE
****************************************************************************/
proc(1)=sc_ll(x);
local x0,p00,p01,p11,p10,ps,D,in,s,eta,r,snum,fy,lambda,gs1,gs2,gams,cp,lam;

x0=trans(x);
x0=constr(x0);
p00=x0[1];
p01=1-p00;
p11=x0[2];
p10=1-p11;
PS=p00~p01|p10~p11;
lambda=x0[3:5];
gams=x0[6:7];
cp=x0[8:9];

if tim==2;
D=(eye(n)-ps')|ones(1,n);
in=zeros(n,1)|1;
s=inv(D'D)*D'in;
else;
s=ps'*fp[.,tim-1];

endif;

eta=zeros(n,1);

   gs1=inv(1+exp(-gams[1]*(xr[tim]-cp[1])));
   gs2=inv(1+exp(-gams[2]*(xr[tim]-cp[2])));
   lam=lambda[1]*gs1+lambda[2]*(1-gs1);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[1]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));
   lam=gs2+lambda[3]*(1-gs2);
  R=SC*lam+eye(nv)*(1-lam); 
       eta[2]=exp(-.5*(ln(det(R))+U[tim,.]*inv(R)*U[tim,.]'@-U[tim,.]*U[tim,.]'@));

   snum=s.*eta;
   fy=ones(1,n)*snum;  
   fy=ln(fy);
retp(fy);
endp; 




/****************************************************************************************************
   PROCEDURE TO REPARAMETERIZE THE COEFFICIENTS
***************************************************************************************************/
proc(1)=trans(x0);
local c,vr,x,y,c1;
x=constr(x0);
if holes==0;
y=exp(x);
else;
y=exp(x);
i=1;
   do while i<=rows(holes);
   y[holes[i]]=0;
   i=i+1;
   endo;
endif;

vr=vech(SC);
vr=delif(vr,vr.==1);
vr=1/maxc(abs(vr));
c=x;
c[1]=y[1]/(1+y[1]);
c[2]=y[2]/(1+y[2]);
c[3]=y[3]/(1+y[3]);
c[4]=c[3]+(y[4])/(1+y[4])*(1-c[3]);
c[5]=1+y[5]/(1+y[5])*(vr-1);
c[6]=y[6];
c[7]=y[7];

if holes==0;
c1=zeros(pmax,1);
else;
c1=zeros(pmax,1);
i=1;
   do while i<=rh;
   c1[holes[i]]=1;
   i=i+1;
   endo;
endif;
c=delif(c,c1);  

retp(c);
endp;

/******************************************************************************
                 PROCEDURE TO CONSTRAINTS SOME PARAMETERS TO BE 0
*******************************************************************************/
proc(1)=constr(x0);
local xc,ir,ix,j;
if holes==0;
xc=x0;
else;
xc=zeros(pmax,1);
ir=1;
ix=0;
   j=1;
   do while j<=pmax;
   ix=ix+1;
      hol:
      if j==holes[ir];
      if holes[ir]==pmax and j==pmax;
      goto ehol1;
      endif;
      j=j+1;
      ix=j-ir;
      ir=ir+1;
      if ir>rh;
      ir=rh;
      goto ehol;
      endif;
      goto hol;
      endif;
   ehol:
   xc[j]=x0[ix];
   ehol1:
   j=j+1;
   endo; 
endif;
retp(xc);
endp;
